﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Parsing;

namespace searchComponent.grammar
{
    /// <summary>
    /// This class represents the numeric Grammar, this is used only for numbers.
    /// Some examples are: 4354, 34.2, 76 or 78, 76:78, 76 and 78, &gt;76, &gt;76 and &lt;78
    /// All numbers will use the . as decimal separator, there are no thousands separator.
    /// </summary>
    public class NumericGrammar : Grammar
    {
        /// <summary>
        /// This is where the grammar is defined.
        /// </summary>
        public NumericGrammar()
        {
            this.GrammarComments = @"
Simple grammar to operate on numbers. It allows numbers and number comparisons like <78
or !54, also it allows and or operators and the : operator which means between. Floats and integers
are allowed, the dot . is the decimal separator.";

            // 1. Terminals
            var number = new NumberLiteral("number");
            number.DefaultFloatType = TypeCode.Double;
            var ImpliedAnd = new ImpliedSymbolTerminal("ImpliedAnd"); 

            // 2. Non Terminals
            var UnaryTerm = new NonTerminal("UnaryTerm");
            var UnaryOp = new NonTerminal("UnaryOp");
            var BinaryTerm = new NonTerminal("BinaryTerm");
            var BinaryOp = new NonTerminal("BinaryOp");
            var Operand = new NonTerminal("Operand");
            var AndExpression = new NonTerminal("AndExpression");
            var AndTerm = new NonTerminal("AndTerm");
            var OrExpression = new NonTerminal("OrExpression");
            var OrOp = new NonTerminal("OrOp");
            var AndOp = new NonTerminal("AndOp");
            var Expression = new NonTerminal("Expression");

            // 3. BNF Rules
            Expression.Rule = OrExpression | AndExpression | Operand;
            OrExpression.Rule = Expression + OrOp + AndTerm;
            AndExpression.Rule = AndTerm + AndOp + Operand;
            AndTerm.Rule = AndExpression | Operand;

            OrOp.Rule = ToTerm("or");
            AndOp.Rule = ToTerm("and") | ImpliedAnd;

            Operand.Rule = UnaryTerm | BinaryTerm;
            UnaryTerm.Rule = (UnaryOp + number) | number;
            BinaryTerm.Rule = number + BinaryOp + number;
            UnaryOp.Rule = ToTerm(">") | "<" | "!";
            BinaryOp.Rule = ToTerm(":");

            this.Root = Expression;

            // 4. Operators precedence
            RegisterOperators(10, OrOp);
            RegisterOperators(20, AndOp);
        }
    }
}
