﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Parsing;
using searchComponent.structure;

namespace searchComponent.interpreter
{
    public abstract class ColumnInterpreter
    {
        /// <summary>
        /// The column that indicates what type of interpretation is needed
        /// </summary>
        protected Column _column;

        /// <summary>
        /// Gets the query needed for this column. THe node is present as a parameter
        /// because the method is recursive.
        /// </summary>
        /// <param name="columnSearch"></param>
        /// <returns></returns>
        public abstract string GetWhereQuery(string columnSearch);

        /// <summary>
        /// Returns a columninterpreter suitable for the type of column passed as parameter
        /// </summary>
        /// <param name="objColumn"></param>
        /// <returns></returns>
        public static ColumnInterpreter GetColumnInterpreter(Column objColumn)
        {
            ColumnInterpreter result = null;
            if (objColumn.Type == Column.ColumnType.Date)
                result = new DateColumnInterpreter(objColumn);
            if (objColumn.Type == Column.ColumnType.Fts)
                result = new FtsColumnInterpreter(objColumn);
            if (objColumn.Type == Column.ColumnType.Numeric)
                result = new NumericColumnInterpreter(objColumn);
            if (objColumn.Type == Column.ColumnType.String)
                result = new StringColumnInterpreter(objColumn);

            return result;
        }
    }
}
